#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
#
# Author:  Valentin Kuznetsov

"""
CMS DBS file manager which find files based on dataset/run/event/branch 
information from DBS.
"""

import os
import sys
import types
import string
import urllib
import urllib2
import elementtree.ElementTree as ET
import traceback

class DBS(object):

  def __init__(self, dbsInst="cms_dbs_prod_global"):
      self.dbs = dbsInst
      self.global_url = "http://cmsdbsprod.cern.ch"
      self.params = {'apiversion': 'DBS_2_0_2', 'api': 'executeQuery',
                     'query': ''}
      self.dbslist = self.dbsinstances()

  def getdbsurl(self,dbs):
      if  dbs.find('tier0') != -1:
          url="http://cmst0dbs.cern.ch"
      else:
          url = self.global_url
      return url + "/%s/servlet/DBSServlet" % dbs

  def dbsinstances(self):
      dbslist = ['cms_dbs_prod_global',
                 'cms_dbs_caf_analysis_01',
                 'cms_dbs_ph_analysis_01',
                 'cms_dbs_ph_analysis_02',
                ]
      for i in range(1,11):
          if  i < 10:
              dbslist.append('cms_dbs_prod_local_0%s' % i)
          else:
              dbslist.append('cms_dbs_prod_local_%s' % i)
      return dbslist

  def blockLookup(self, lfn):
      """
      Return the block location of a particular LFN.
      """
      if  lfn.find("*")!=-1:
          msg = "Wrong lfn format, %s"%lfn
          raise msg
      query  = "find block where file=%s"%lfn
      self.params['query'] = query
      dbsurl = self.getdbsurl(self.dbs)
      data = urllib2.urlopen(dbsurl, urllib.urlencode(self.params,
          doseq=True)).read()
      blockList = parseDBSoutput(data)
      if  len(blockList)>1:
          msg = "LFN %s found in more then 1 block %s"%(lfn,str(blockList))
          raise msg
      return blockList[0]

  def getTiers(self):
      """
      Return list of tiers registered in DBS
      """
#      query  = "find tier where tier like *"
      query = "find dataset.tier"
      self.params['query'] = query
      dbsurl = self.getdbsurl(self.dbs)
      data = urllib2.urlopen(dbsurl, urllib.urlencode(self.params,
          doseq=True)).read()
      tierList = parseDBSoutput(data)
      return tierList

  def getFiles(self, run=None, dataset=None, evt=None, branchList=None):
      """
      Return list of lfns for given run/(dataset,evt,branch)
      """
      query = "find file,file.size where"
      cond = ""
      # condition on run
      if  run :
          run = str(run)
          if run.find("*") != -1:
              cond += " and run like %s" % run
          else:
              cond += " and run=%s" % run
      # condition on dataset
      if  dataset:
          if dataset.find("*") != -1:
              cond += " and dataset like %s" % dataset
          else:
              cond += " and dataset=%s" % dataset
      # condition on event
      if evt:
          if type(evt) is types.TupleType or type(evt) is types.ListType:
              cond += " and run.startofrun=%s and run.endofrun=%s " % \
                  (evt[0], evt[1])
          else:
              cond += " and lumi.startevnum <= %s and lumi.endevnum >= %s " % \
                  (evt, evt)
      if  branchList:
          for b in branchList:
              cond += " and file.branch = %s" % b
      if  not cond:
          msg  = "Unable to build query, not sufficient clause condition:\n"
          msg += "run=%s, dataset=%s, evt=%s, branch=%s" \
                % (run,dataset,evt,branchList)
          print msg
          raise RuntimeError
      query = "find file,file.size where %s" % cond[4:] # don't use first and
      print query
      self.params['query'] = query
      fileList = []
      for dbs in self.dbslist:
          dbsurl = self.getdbsurl(dbs)
          data = urllib2.urlopen(dbsurl, 
                         urllib.urlencode(self.params, doseq=True)).read()
          fileList = parseDBSoutput(data)
          if  fileList:
              return fileList
      return fileList

  def getFilesForRun(self, run):
      """
      Return list of lfns for given run
      """
      return self.getFiles(run)
      
def parseDBSoutput(data):
    elem  = ET.fromstring(data)
    oList = []
    for i in elem:
        if i.tag=="result":
           if i.attrib.has_key('FILES_FILESIZE') and \
              i.attrib.has_key('FILES_LOGICALFILENAME'):
              oList.append((i.attrib['FILES_LOGICALFILENAME'],
                            sizeFormat(i.attrib['FILES_FILESIZE']) ))
           elif i.attrib.has_key('FILES_LOGICALFILENAME'):
              oList.append(i.attrib['FILES_LOGICALFILENAME'])
           elif i.attrib.has_key('DATATIER_NAME'):
              oList.append(i.attrib['DATATIER_NAME'])
           elif i.attrib.has_key('BLOCK_NAME'):
              oList.append(i.attrib['BLOCK_NAME'])
    return oList

def sizeFormat(i):
    """
    Format file size utility, it converts file size into KB, MB, GB, TB, PB
    units
    """
    try:
       num = long(i)
    except:
       return "N/A"
    for x in ['', 'KB', 'MB', 'GB', 'TB', 'PB']:
        if num < 1024.:
            return "%3.1f%s" % (num, x)
        num /= 1024.


def dbsinst():
    # DBS imports
    from   RS.Wrapper import RegService
    dbslist = []
    try :
        api = RegService()
        result = api.queryRegistrationFindAll()
        for i in result:
            dbslist.append( (i._alias, i._url) )
        #result = api.queryRegistrationFindByAlias("Prod_Global")
        #for i in result:
                #printReg(i)
    except Exception, ex:
        traceback.print_exc()
        raise
    return dbslist

#
# main
#
def main():
    print "TEST DBS RS"
    try:
        print dbsinst()
    except:
        traceback.print_exc()
    dbsInst = 'cms_dbs_prod_global'
    dbscls = DBS(dbsInst)
    try:
        dbscls.getFiles() # should fail
    except:
        pass
    run = 16288
    dataset = "/testbeam_HCalEcalCombined/h2tb2007_default_v1/DIGI-RECO"
    evt = 44000
    print "Files for run", run
    print dbscls.getFiles(run)
    print "Files for run %s, dataset %s, event %s." % (run, dataset, evt)
    print dbscls.getFiles(run, dataset, evt)
    evt = 10
    print "Files for run %s, dataset %s, event %s." % (run, dataset, evt)
    print dbscls.getFiles(run, dataset, evt)
    print dbscls.getTiers()
    lfn="/store/data/h2tb2007/testbeam_HCalEcalCombined/RAW/default_v1/h2.00016290.0007.edm.storageManager.0.0000.root"
    print dbscls.blockLookup(lfn)

    import time
    t = time.time()
    dataset = "/dataset_PD_3/BUNKACQUISITIONERA-v1/RAW"
    flist = dbscls.getFiles(run="", dataset=dataset, evt="")
    print flist[:5]
    print time.time()-t

if __name__ == "__main__":
    main()
