
import os

import FileMover

class EdmFileExtraction(ConfiguredObject):

    """
    Use the external program, EdmFileUtil, to extract a (run, event) from one
    of the possible LFNs into a separate file.

    This should save the event into the file
       /$dest_dir/$run/$event
    """

    def __init__(self, cp):
        super(EdmFileExtraction, self).__init__(cp)
        self.cp = cp
        self.section = "edm_file_extraction"
        self._lfns = None
        self._run = None
        self._event = None
        self._src_dir = None
        self._dest_dir = None

    def request(self, run, event, lfns, src_dir, dest_dir):
        """
        Request that the (run, event) be extracted from the LFNs in src_dir,
        and results written into dest_dir
        """
        self._lfns = lfns
        self._run = run
        self._event = event
        self._src_dir = src_dir
        self._dest_dir = dest_dir

    def getResults(self):
        """
        If status returns code == 0, then getResults will return the absolute
        path to the contents of the event
        """
        #TODO: Check to make sure we actually have finished successfully.
        return os.path.join(self._dest_dir, self._run, self._event)

    def start(self):
        """
        Perform the necessary extraction, saving things to whatever
        directory we were assigned.
        """
        # TODO: Make sure we check the cache first.
        raise NotImplementedError()

    def cancel(self):
        """
        Cancel extraction.
        """
        raise NotImplementedError()

    def status(self):
        """
        Return the status of the object.
        """
        raise NotImplementedError()

